// ManifKeypad.java
// 01.03.96
//
// the keypad on the manifestations screen

package cybcerone.manif;

import java.awt.Event;

import cybcerone.utils.Appletlike;
import cybcerone.utils.DualKeypad;
import cybcerone.utils.KeypadButton;

/**
 * Pretty much like all the other keypads, except two buttons can be
 * pushed at once.
 */
class ManifKeypad extends DualKeypad {
  private static final String id = "manifKeypad";
  private static final String statusText = "Select period and department";

  private static final String bgFile =
    ManifPanel.imagePath + "fond_manif.gif";
  private static final String activeBgFile =
    ManifPanel.imagePath + "fond_manif_rouge.gif";

  private Interval today = new Interval (Interval.TODAY);
  private Interval thisWeek = new Interval (Interval.THISWEEK);
  private Interval nextWeeks = new Interval (Interval.NEXTWEEKS);
  private Interval all = new Interval (Interval.ALL);

  ManifKeypad (Appletlike app) {
    super (id, statusText, app);

    setBackground (bgFile, 6);
    setActiveBackground (activeBgFile, 6);

    makeButtons ();
    reset ();
  }

  public void reset () {
    update (whichButton (scale (45), scale (215))); /* today interval button */
    update (whichButton (scale (109), scale (395))); /* all facultes button */
  }

  private void makeButtons () {
    addButton (today, 35, 205, 220, 42, true);
    addButton (thisWeek, 35, 248, 220, 42, true);
    addButton (nextWeeks, 35, 292, 220, 42, true);
    addButton (all, 0, 338, 190, 50, true);

    addButton (new Fac (Fac.ALL), 99, 385, 190, 50, false);
    addButton (new Fac (Fac.THEOLOGIE), 35, 439, 106, 40, false);
    addButton (new Fac (Fac.DROIT), 150, 439, 106, 40, false);
    addButton (new Fac (Fac.LETTRES), 35, 483, 106, 40, false);
    addButton (new Fac (Fac.SSP), 150, 483, 106, 40, false);
    addButton (new Fac (Fac.HEC), 35, 526, 106, 40, false);
    addButton (new Fac (Fac.SCIENCES), 150, 526, 106, 40, false);
    addButton (new Fac (Fac.MEDECINE), 35, 571, 106, 40, false);
    addButton (new Fac (Fac.SERVICES), 150, 571, 106, 40, false);
    addButton (new Fac (Fac.AUTRES), 35, 616, 220, 40, false);
  }

  public boolean mouseMove (Event evt, int x, int y) {
    KeypadButton theButton = whichButton (x, y);

    if (theButton != null) {
      Object value = theButton.getValue ();

      if (value.equals (today))
	app.showStatus ("Today");
      else if (value.equals (thisWeek))
	app.showStatus ("This week");
      else if (value.equals (nextWeeks))
	app.showStatus ("Following weeks");
      else if (value.equals (all))
	app.showStatus ("All the periods");
      else
	app.showStatus (statusText);
    } else
      app.showStatus (statusText);
    
    return true;
  }
  
}
